@@ -31,9 +31,8 @@
 /********************************************************/
 
 int appmenu_exitcallback_2(int menuid, bool exitall) {
-    if ((menuid == MENU_BASE
-                || menuid == MENU_BASE_A_THRU_F
-                || menuid == MENU_BASE_LOGIC)
+    if (menuid >= MENU_BASE
+            && menuid <= MENU_BASE_LOGIC
             && !exitall) {
         mode_appmenu = menuid;
         set_appmenu_exitcallback(2);
@@ -118,7 +117,9 @@ int docmd_and(arg_struct *arg) {
         return err;
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
-    v = new_real((phloat) (x & y));
+    int8 res = x & y;
+    base_range_check(&res, true);
+    v = new_real(base2phloat(res));
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -134,9 +135,34 @@ int docmd_baseadd(arg_struct *arg) {
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
     res = x + y;
-    if ((err = base_range_check(&res)) != ERR_NONE)
-        return err;
-    v = new_real((phloat) res);
+    if (!flags.f.base_wrap && effective_wsize() == 64) {
+        if (flags.f.base_signed) {
+            if (x > 0 && y > 0) {
+                if (res < x || res < y)
+                    if (flags.f.range_error_ignore)
+                        res = (1LL << (effective_wsize() - 1)) - 1;
+                    else
+                        return ERR_OUT_OF_RANGE;
+            } else if (x < 0 && y < 0) {
+                if (res > x || res > y)
+                    if (flags.f.range_error_ignore)
+                        res = -1LL << (effective_wsize() - 1);
+                    else
+                        return ERR_OUT_OF_RANGE;
+            }
+        } else {
+            if ((uint8) res < (uint8) x || (uint8) res < (uint8) y)
+                if (flags.f.range_error_ignore)
+                    res = ~0LL;
+                else
+                    return ERR_OUT_OF_RANGE;
+        }
+    } else {
+        err = base_range_check(&res, false);
+        if (err != ERR_NONE)
+            return err;
+    }
+    v = new_real(base2phloat(res));
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -151,10 +177,36 @@ int docmd_basesub(arg_struct *arg) {
         return err;
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
-    res = y - x;
-    if ((err = base_range_check(&res)) != ERR_NONE)
-        return err;
-    v = new_real((phloat) res);
+    if (!flags.f.base_signed && !flags.f.base_wrap && (uint8) x > (uint8) y) {
+        if (flags.f.range_error_ignore)
+            res = 0;
+        else
+            return ERR_OUT_OF_RANGE;
+    } else {
+        res = y - x;
+        if (!flags.f.base_wrap && effective_wsize() == 64) {
+            if (flags.f.base_signed) {
+                if (x < 0 && y > 0) {
+                    if (x == (int8) 0x8000000000000000LL || res < -x || res < y)
+                        if (flags.f.range_error_ignore)
+                            res = (1LL << (effective_wsize() - 1)) - 1;
+                        else
+                            return ERR_OUT_OF_RANGE;
+                } else if (x > 0 && y < 0) {
+                    if (y == (int8) 0x8000000000000000LL || res > -x || res > y)
+                        if (flags.f.range_error_ignore)
+                            res = -1LL << (effective_wsize() - 1);
+                        else
+                            return ERR_OUT_OF_RANGE;
+                }
+            }
+        } else {
+            err = base_range_check(&res, false);
+            if (err != ERR_NONE)
+                return err;
+        }
+    }
+    v = new_real(base2phloat(res));
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -163,30 +215,108 @@ int docmd_basesub(arg_struct *arg) {
 
 int docmd_basemul(arg_struct *arg) {
     int8 x, y;
-    double res;
+    int8 res;
     int err;
     vartype *v;
     if ((err = get_base_param(reg_x, &x)) != ERR_NONE)
         return err;
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
-    /* I compute the result in 'double' arithmetic, because doing it
-     * in int8 arithmetic could cause me to overlook an out-of-range
-     * condition (e.g. 2^32 * 2^32).
-     */
-    res = ((double) x) * ((double) y);
-    if (res < -34359738368.0) {
-        if (flags.f.range_error_ignore)
-            res = -34359738368.0;
-        else
-            return ERR_OUT_OF_RANGE;
-    } else if (res > 34359738367.0) {
-        if (flags.f.range_error_ignore)
-            res = 34359738367.0;
-        else
-            return ERR_OUT_OF_RANGE;
+    if (x == 0 || y == 0) {
+        res = 0;
+    } else {
+        /* Performing a 64-bit x 64-bit => 128-bit multiplication.
+         * Doing it unsigned to keep things somewhat simple.
+         */
+        int wsize = effective_wsize();
+        bool neg = false;
+        uint8 op1, op2;
+        if (flags.f.base_signed) {
+            if (wsize == 64) {
+                if (x == (int8) 0x8000000000000000LL && y != 1) {
+                    if (flags.f.base_wrap)
+                        res = (y & 1) == 0 ? 0 : x;
+                    else if (flags.f.range_error_ignore)
+                        res = y > 0 ? 0x8000000000000000LL : 0x7fffffffffffffffLL;
+                    else
+                        return ERR_OUT_OF_RANGE;
+                } else if (y == (int8) 0x8000000000000000LL && x != 1) {
+                    if (flags.f.base_wrap)
+                        res = (x & 1) == 0 ? 0 : y;
+                    else if (flags.f.range_error_ignore)
+                        res = x > 0 ? 0x8000000000000000LL : 0x7fffffffffffffffLL;
+                    else
+                        return ERR_OUT_OF_RANGE;
+                }
+            }
+            if (x < 0) {
+                neg = true;
+                op1 = (uint8) -x;
+            } else {
+                op1 = (uint8) x;
+            }
+            if (y < 0) {
+                neg = !neg;
+                op2 = (uint8) -y;
+            } else {
+                op2 = (uint8) y;
+            }
+        } else {
+            op1 = (uint8) x;
+            op2 = (uint8) y;
+        }
+
+        uint8 u1 = op1 & 0xffffffff;
+        uint8 v1 = op2 & 0xffffffff;
+        uint8 t = u1 * v1;
+        uint8 w3 = t & 0xffffffff;
+        uint8 k = t >> 32;
+
+        op1 >>= 32;
+        t = op1 * v1 + k;
+        k = t & 0xffffffff;
+        uint8 w1 = t >> 32;
+
+        op2 >>= 32;
+        t = u1 * op2 + k;
+        k = t >> 32;
+
+        uint8 hi = op1 * op2 + w1 + k;
+        uint8 lo = (t << 32) + w3;
+
+        if (flags.f.base_wrap) {
+            if (neg)
+                res = -(int8) lo;
+            else
+                res = (int8) lo;
+            base_range_check(&res, true);
+        } else if (flags.f.base_signed) {
+            if (neg) {
+                if (hi != 0 || lo > (1ULL << (wsize - 1)))
+                    if (flags.f.range_error_ignore)
+                        lo = 1ULL << (wsize - 1);
+                    else
+                        return ERR_OUT_OF_RANGE;
+                res = - (int8) lo;
+            } else {
+                if (hi != 0 || lo > (1ULL << (wsize - 1)) - 1)
+                    if (flags.f.range_error_ignore)
+                        lo = (1ULL << (wsize - 1)) - 1;
+                    else
+                        return ERR_OUT_OF_RANGE;
+                res = (int8) lo;
+            }
+        } else {
+            if (hi != 0 || wsize < 64 && lo > (1ULL << wsize) - 1)
+                if (flags.f.range_error_ignore)
+                    lo = wsize == 64 ? ~0ULL : (1ULL << wsize) - 1;
+                else
+                    return ERR_OUT_OF_RANGE;
+            res = (int8) lo;
+        }
     }
-    v = new_real(res);
+
+    v = new_real(base2phloat(res));
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -203,10 +333,13 @@ int docmd_basediv(arg_struct *arg) {
         return err;
     if (x == 0)
         return ERR_DIVIDE_BY_0;
-    res = y / x;
-    if ((err = base_range_check(&res)) != ERR_NONE)
+    if (flags.f.base_signed)
+        res = y / x;
+    else
+        res = (int8) (((uint8) y) / ((uint8) x));
+    if ((err = base_range_check(&res, false)) != ERR_NONE)
         return err;
-    v = new_real((phloat) res);
+    v = new_real(base2phloat(res));
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -218,14 +351,28 @@ int docmd_basechs(arg_struct *arg) {
     int err;
     if ((err = get_base_param(reg_x, &x)) != ERR_NONE)
         return err;
-    if (x == LL(-34359738368)) {
-        if (flags.f.range_error_ignore)
-            x = LL(34359738367);
-        else
-            return ERR_OUT_OF_RANGE;
-    } else
+    if (flags.f.base_wrap) {
         x = -x;
-    ((vartype_real *) reg_x)->x = (phloat) x;
+        base_range_check(&x, true);
+    } else if (flags.f.base_signed) {
+        int8 maxneg = -1LL << (effective_wsize() - 1);
+        if (x == maxneg) {
+            if (flags.f.range_error_ignore)
+                x = -1 - maxneg;
+            else
+                return ERR_OUT_OF_RANGE;
+        } else
+            x = -x;
+    } else {
+        if (x != 0) {
+            if (flags.f.range_error_ignore)
+                x = 0;
+            else
+                return ERR_OUT_OF_RANGE;
+        } else
+            x = 0;
+    }
+    ((vartype_real *) reg_x)->x = base2phloat(x);
     return ERR_NONE;
 }
 
@@ -453,9 +600,9 @@ int docmd_bit_t(arg_struct *arg) {
         return err;
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
-    if (x < 0 || x > 35)
+    if (x < 0 || x >= effective_wsize())
         return ERR_INVALID_DATA;
-    return (y & (LL(1) << x)) != 0 ? ERR_YES : ERR_NO;
+    return (y & (1ULL << x)) != 0 ? ERR_YES : ERR_NO;
 }
 
 int docmd_corr(arg_struct *arg) {
@@ -722,7 +869,9 @@ int docmd_not(arg_struct *arg) {
     vartype *v;
     if ((err = get_base_param(reg_x, &x)) != ERR_NONE)
         return err;
-    v = new_real((phloat) ~x);
+    int8 res = ~x;
+    base_range_check(&res, true);
+    v = new_real(base2phloat(res));
     if (v == NULL)
         return ERR_INSUFFICIENT_MEMORY;
     unary_result(v);
@@ -737,7 +886,9 @@ int docmd_or(arg_struct *arg) {
         return err;
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
-    v = new_real((phloat) (x | y));
+    int8 res = x | y;
+    base_range_check(&res, true);
+    v = new_real(base2phloat(res));
     if (v == NULL) 
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -856,31 +1007,40 @@ int docmd_pgminti(arg_struct *arg) {
 }
 
 int docmd_rotxy(arg_struct *arg) {
-    int8 x, y, res;
+    int x;
+    uint8 y, res;
     int err; 
     vartype *v;
-    if ((err = get_base_param(reg_x, &x)) != ERR_NONE) 
-        return err;
-    if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
-        return err;
-    if (x < -35 || x > 35)
+
+    // Not using get_base_param() to fetch x, because that
+    // would make it impossible to specify a negative shift
+    // count in unsigned mode.
+    if (reg_x->type == TYPE_STRING)
+        return ERR_ALPHA_DATA_IS_INVALID;
+    else if (reg_x->type != TYPE_REAL)
+        return ERR_INVALID_TYPE;
+    phloat px = floor(((vartype_real *) reg_x)->x);
+    int wsize = effective_wsize();
+    if (px >= wsize || px <= -wsize)
         return ERR_INVALID_DATA;
+    x = to_int(px);
+
+    if ((err = get_base_param(reg_y, (int8 *) &y)) != ERR_NONE)
+        return err;
     if (x == 0)
         res = y;
     else {
-        y &= LL(0xfffffffff);
+        if (wsize < 64)
+            y &= (1ULL << wsize) - 1;
         if (x > 0)
-            res = (y >> x) | (y << (36 - x));
+            res = (y >> x) | (y << (wsize - x));
         else {
             x = -x;
-            res = (y << x) | (y >> (36 - x));
+            res = (y << x) | (y >> (wsize - x));
         }
-        if ((res & LL(0x800000000)) == 0)
-            res &= LL(0x7ffffffff);
-        else
-            res |= LL(0xfffffff000000000);
+        base_range_check((int8 *) &res, true);
     }
-    v = new_real((phloat) res);
+    v = new_real(base2phloat((int8) res));
     if (v == NULL) 
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
@@ -956,7 +1116,9 @@ int docmd_xor(arg_struct *arg) {
         return err;
     if ((err = get_base_param(reg_y, &y)) != ERR_NONE)
         return err;
-    v = new_real((phloat) (x ^ y));
+    int8 res = x ^ y;
+    base_range_check(&res, true);
+    v = new_real(base2phloat(res));
     if (v == NULL) 
         return ERR_INSUFFICIENT_MEMORY;
     binary_result(v);
